﻿using System;
using System.Collections.Generic;
using System.Net;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MiniProxy;

namespace UT
{
    [TestClass]
    public class MiniProxyTest
    {
        class MyHttpHeaderTracer : IProxyHook
        {
            public Dictionary<string, object> reqHeaders = new Dictionary<string, object>();
            
            public void HookRequest(HttpListenerRequest req)
            {
                foreach(string h in req.Headers.Keys)
                {
                    reqHeaders.Add(h, req.Headers[h]);
                }
            }

            public void HookResponse(HttpWebResponse resp)
            {
            }
        }

        [TestMethod]
        public void TestRequestHeaderTracing()
        {
            MyHttpHeaderTracer myHeaderTracer = new MyHttpHeaderTracer();
            var tracy = new MiniProxy.MiniProxy(myHeaderTracer);
            tracy.Start(9876);

            HttpWebRequest req = (HttpWebRequest)WebRequest.Create("http://codeplex.com");
            req.Referer = "referee";
            req.Headers["my"] = "qwerty";
            req.Proxy = new WebProxy("127.0.0.1", 9876);
            WebResponse resp = req.GetResponse();
            foreach (string h in resp.Headers.Keys)
            {
                Console.WriteLine("", h, resp.Headers[h]);
            }

            // verify the headers inside myHeaderTracer
            Assert.AreEqual("qwerty", myHeaderTracer.reqHeaders["my"]);
            tracy.Stop();

        }

        [TestMethod]
        public void TestStop()
        {
            MyHttpHeaderTracer myHeaderTracer = new MyHttpHeaderTracer();
            var tracy = new MiniProxy.MiniProxy(myHeaderTracer);
            tracy.Start(9876);
            tracy.Stop();

            bool exceptionThrown = false;
            try
            {
                HttpWebRequest req = (HttpWebRequest)WebRequest.Create("http://odata.org");
                req.Referer = "referee";
                req.Headers["my"] = "qwerty";
                req.Proxy = new WebProxy("127.0.0.1", 9876);
                WebResponse resp = req.GetResponse();
            }
            catch (WebException)
            {
                exceptionThrown = true;
            }

            Assert.IsTrue(exceptionThrown);
        }
    }
}
